/**
 * BladeX Commercial License Agreement
 * Copyright (c) 2018-2099, https://bladex.cn. All rights reserved.
 * <p>
 * Use of this software is governed by the Commercial License Agreement
 * obtained after purchasing a license from BladeX.
 * <p>
 * 1. This software is for development use only under a valid license
 * from BladeX.
 * <p>
 * 2. Redistribution of this software's source code to any third party
 * without a commercial license is strictly prohibited.
 * <p>
 * 3. Licensees may copyright their own code but cannot use segments
 * from this software for such purposes. Copyright of this software
 * remains with BladeX.
 * <p>
 * Using this software signifies agreement to this License, and the software
 * must not be used for illegal purposes.
 * <p>
 * THIS SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY. The author is
 * not liable for any claims arising from secondary or illegal development.
 * <p>
 * Author: Chill Zhuang (bladejava@qq.com)
 */
package org.springblade.modules.spzf.payInfo.controller;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.alibaba.fastjson.TypeReference;
import com.alibaba.fastjson.parser.Feature;
import com.alipay.api.AlipayApiException;
import com.alipay.api.internal.util.AlipayEncrypt;
import com.alipay.api.internal.util.AlipaySignature;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.Parameter;
import io.swagger.v3.oas.annotations.tags.Tag;
import jakarta.servlet.http.HttpServletResponse;
import jakarta.validation.Valid;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springblade.core.boot.ctrl.BladeController;
import org.springblade.core.excel.util.ExcelUtil;
import org.springblade.core.http.util.HttpUtil;
import org.springblade.core.mp.support.Condition;
import org.springblade.core.mp.support.Query;
import org.springblade.core.secure.BladeUser;
import org.springblade.core.secure.annotation.PreAuth;
import org.springblade.core.tool.api.R;
import org.springblade.core.tool.constant.RoleConstant;
import org.springblade.core.tool.utils.DateUtil;
import org.springblade.core.tool.utils.Func;
import org.springblade.modules.spzf.payInfo.excel.PayInfoExcel;
import org.springblade.modules.spzf.payInfo.pojo.entity.PayInfoEntity;
import org.springblade.modules.spzf.payInfo.pojo.vo.PayInfoVO;
import org.springblade.modules.spzf.payInfo.pojo.vo.wx.PhoneNumberVO;
import org.springblade.modules.spzf.payInfo.service.IPayInfoService;
import org.springblade.modules.spzf.payInfo.wrapper.PayInfoWrapper;
import org.springblade.modules.system.pojo.entity.User;
import org.springblade.modules.system.service.IUserService;
import org.springframework.web.bind.annotation.*;

import java.util.List;
import java.util.Map;

/**
 * 付款记录 控制器
 *
 * @author wth
 * @since 2024-12-06
 */
@Slf4j
@RestController
@AllArgsConstructor
@RequestMapping("/payInfo")
@Tag(name = "付款记录", description = "付款记录接口")
public class PayInfoController extends BladeController {

	private final IPayInfoService payInfoService;

	private final String appid = "wxade26aebf348223e";

	private final String secret = "5ce4a147f3cfe7b6eb4953b41109ce66";

	private final IUserService userService;

	/**
	 * 微信登录
	 * params js_code code
	 */
	@GetMapping("/wxLogin")
	public R wxLogin(@RequestParam Map<String, Object> params) {
		log.info("参数js_code打印：" + params.get("js_code").toString());
		log.info("参数code打印：" + params.get("code").toString());
		// 小程序登录接口 参数：appid/secret/js_code/grant_type
		params.put("appid", appid);
		params.put("secret", secret);
		params.put("grant_type", "authorization_code");
		String code2SessionResponse = HttpUtil.get("https://api.weixin.qq.com/sns/jscode2session", params);
		JSONObject code2SessionJsonObject = JSONObject.parseObject(code2SessionResponse);
		if (null != code2SessionJsonObject.get("errcode")) {
			return R.fail("登录失败");
		}
		String openid = code2SessionJsonObject.getString("openid");
		// 根据openid查询blade_user表，如果没有，需要通过微信接口获取手机号
		QueryWrapper<User> queryWrapper = new QueryWrapper<>();
		queryWrapper.eq("account", openid);
		User user = userService.getOne(queryWrapper);
		if (null != user) {
			return R.data(user.getPhone());
		} else {
			// 获取接口调用凭据(access_token) 参数：grant_type/appid/secret
			params.put("grant_type", "client_credential");
			JSONObject getAccessTokenJsonObject = JSONObject.parseObject(HttpUtil.get("https://api.weixin.qq.com/cgi-bin/token", params));
			String accessToken = getAccessTokenJsonObject.getString("access_token");
			// 获取手机号 参数：grant_type
			PhoneNumberVO phoneNumberVO = JSONObject.parseObject(HttpUtil.postJson("https://api.weixin.qq.com/wxa/business/getuserphonenumber?access_token=" + accessToken, JSONObject.toJSONString(params.get("code"))), PhoneNumberVO.class);
			// 注册到blade_user
			User newUser = new User();
			newUser.setAccount(openid);
			newUser.setPhone(phoneNumberVO.getPhone_info().getPhoneNumber());
			newUser.setCode("wx");
			newUser.setPassword("Xwkj2024@ 确认加密后的密码");
			userService.submit(newUser);
			return R.data(phoneNumberVO.getPhone_info().getPhoneNumber());
		}
	}

	/**
	 * 支付宝 验签与解密处理
	 * params js_code code
	 */
	@GetMapping("/zfbLogin")
	public R zfbLogin(@RequestParam Map<String, Object> params) throws Exception {
		String response = "小程序前端返回的加密信息";
		//1. 获取验签和解密所需要的参数
		Map<String, String> openapiResult = JSON.parseObject(response,new TypeReference<Map<String, String>>() {}, Feature.OrderedField);
		String signType = "RSA2";
		String charset = "UTF-8";
		String encryptType = "AES";
		String sign = openapiResult.get("sign");
		String content = openapiResult.get("response");
		//判断是否为加密内容
		boolean isDataEncrypted = !content.startsWith("{");
		boolean signCheckPass = false;
		//2. 验签
		String signContent = content;
		String signVeriKey = "你的小程序对应的支付宝公钥（为扩展考虑建议用appId+signType做密钥存储隔离）";
		String decryptKey = "你的小程序对应的加解密密钥（为扩展考虑建议用appId+encryptType做密钥存储隔离）";//如果是加密的报文则需要在密文的前后添加双引号
		if (isDataEncrypted) {
			signContent = "\"" + signContent + "\"";
		} try {
			signCheckPass = AlipaySignature.rsaCheck(signContent, sign, signVeriKey, charset, signType);
		} catch (AlipayApiException e) {
			// 验签异常, 日志
		} if (!signCheckPass) {
			//验签不通过（异常或者报文被篡改），终止流程（不需要做解密）
			throw new Exception("验签失败");
		}
		//3. 解密
		String plainData = null;
		if (isDataEncrypted) {
			try {
				plainData = AlipayEncrypt.decryptContent(content, encryptType, decryptKey, charset);
			} catch (AlipayApiException e) {
				//解密异常, 记录日志
				throw new Exception("解密异常");
			}} else {
			plainData = content;
		}
		return null;
	}

	/**
	 * 付款记录 详情
	 */
	@GetMapping("/detail")
	@ApiOperationSupport(order = 1)
	@Operation(summary = "详情", description  = "传入payInfo")
	public R<PayInfoVO> detail(PayInfoEntity payInfo) {
		PayInfoEntity detail = payInfoService.getOne(Condition.getQueryWrapper(payInfo));
		return R.data(PayInfoWrapper.build().entityVO(detail));
	}

	/**
	 * 付款记录 分页
	 */
	@GetMapping("/list")
	@ApiOperationSupport(order = 2)
	@Operation(summary = "分页", description  = "传入payInfo")
	public R<IPage<PayInfoVO>> list(@Parameter(hidden = true) @RequestParam Map<String, Object> payInfo, Query query) {
		IPage<PayInfoEntity> pages = payInfoService.page(Condition.getPage(query), Condition.getQueryWrapper(payInfo, PayInfoEntity.class));
		return R.data(PayInfoWrapper.build().pageVO(pages));
	}


	/**
	 * 付款记录 自定义分页
	 */
	@GetMapping("/page")
	@ApiOperationSupport(order = 3)
	@Operation(summary = "分页", description  = "传入payInfo")
	public R<IPage<PayInfoVO>> page(PayInfoVO payInfo, Query query) {
		IPage<PayInfoVO> pages = payInfoService.selectPayInfoPage(Condition.getPage(query), payInfo);
		return R.data(pages);
	}

	/**
	 * 付款记录 新增
	 */
	@PostMapping("/save")
	@ApiOperationSupport(order = 4)
	@Operation(summary = "新增", description  = "传入payInfo")
	public R save(@Valid @RequestBody PayInfoEntity payInfo) {
		return R.status(payInfoService.save(payInfo));
	}

	/**
	 * 付款记录 修改
	 */
	@PostMapping("/update")
	@ApiOperationSupport(order = 5)
	@Operation(summary = "修改", description  = "传入payInfo")
	public R update(@Valid @RequestBody PayInfoEntity payInfo) {
		return R.status(payInfoService.updateById(payInfo));
	}

	/**
	 * 付款记录 新增或修改
	 */
	@PostMapping("/submit")
	@ApiOperationSupport(order = 6)
	@Operation(summary = "新增或修改", description  = "传入payInfo")
	public R submit(@Valid @RequestBody PayInfoEntity payInfo) {
		return R.status(payInfoService.saveOrUpdate(payInfo));
	}

	/**
	 * 付款记录 删除
	 */
	@PostMapping("/remove")
	@ApiOperationSupport(order = 7)
	@Operation(summary = "逻辑删除", description  = "传入ids")
	public R remove(@Parameter(description = "主键集合", required = true) @RequestParam String ids) {
		return R.status(payInfoService.deleteLogic(Func.toLongList(ids)));
	}

	/**
	 * 导出数据
	 */
	@PreAuth(RoleConstant.HAS_ROLE_ADMIN)
	@GetMapping("/export-payInfo")
	@ApiOperationSupport(order = 8)
	@Operation(summary = "导出数据", description  = "传入payInfo")
	public void exportPayInfo(@Parameter(hidden = true) @RequestParam Map<String, Object> payInfo, BladeUser bladeUser, HttpServletResponse response) {
		QueryWrapper<PayInfoEntity> queryWrapper = Condition.getQueryWrapper(payInfo, PayInfoEntity.class);
		//if (!AuthUtil.isAdministrator()) {
		//	queryWrapper.lambda().eq(PayInfoEntity::getTenantId, bladeUser.getTenantId());
		//}
		//queryWrapper.lambda().eq(PayInfoEntity::getIsDeleted, BladeConstant.DB_NOT_DELETED);
		List<PayInfoExcel> list = payInfoService.exportPayInfo(queryWrapper);
		ExcelUtil.export(response, "付款记录数据" + DateUtil.time(), "付款记录数据表", list, PayInfoExcel.class);
	}

}
